<?php

/**
 * 2007-2022 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2022 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

if (!defined("AP_PATH_DIR")) {
    define("AP_PATH_DIR", dirname(__FILE__));
}

require_once _PS_MODULE_DIR_ . "aplazo/src/loader.php";

class Aplazo extends PaymentModule
{
    protected $config_form = false;

    public function __construct()
    {
        $this->name = 'aplazo';
        $this->tab = 'payments_gateways';
        $this->version = '1.6.3';
        $this->author = 'Conexa';
        $this->need_instance = true;
        $this->module_key = '129f989e8dab50b04ef31b15c94b9aea';

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Aplazo');
        $this->description = $this->l(
            'Allow your clients to pay in installments in a simple and easy way.'
        );
        $this->limited_countries = ["MX"];
        $this->limited_currencies = ["MXN"];
        $this->ps_versions_compliancy = array(
            'min' => '1.6', 'max' => _PS_VERSION_
        );
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');
            return false;
        }

        $iso_code = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));

        if (in_array($iso_code, $this->limited_countries) == false) {
            $this->_errors[] = $this->l('This module is not available in your country');
            return false;
        }

        include(dirname(__FILE__) . "/sql/install.php");
        include(dirname(__FILE__) . "/sql/upgrade-1.4.0.php");

        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }

        return parent::install() &&
            $this->installTab() &&
            $this->registerHook('header') &&
            $this->registerHook('backOfficeHeader') &&
            $this->registerHook('payment') &&
            // $this->registerHook('displayAdminOrder') &&
            $this->registerHook('displayAdminOrderRight') &&
            $this->registerHook('displayAdminOrderSide') &&
            $this->registerHook('paymentOptions');
    }

    public function uninstall()
    {
        Configuration::deleteByName('APLAZO_API_TOKEN');
        Configuration::deleteByName('APLAZO_MERCHANT_ID');
        Configuration::deleteByName('APLAZO_TOKEN');
        Configuration::deleteByName('APLAZO_TOKEN_SUCCESS');
        Configuration::deleteByName('CONF_APLAZO_FIXED');
        Configuration::deleteByName('CONF_APLAZO_FIXED_FOREIGN');
        Configuration::deleteByName('CONF_APLAZO_VAR');
        Configuration::deleteByName('CONF_APLAZO_VAR_FOREIGN');

        include(dirname(__FILE__) . "/sql/uninstall.php");
        return parent::uninstall();
    }

    /**
     * @return bool
     */
    private function installTab(): bool
    {
        $tab = new Tab();
        $tab->name = [];
        $tab->name[1] = 'Aplazo Cancel';
        $tab->name[2] = 'Aplazo Cancel';
        $tab->active = 1;
        $tab->class_name = 'CancelAplazoController';
        $tab->module = $this->name;
        return $tab->save();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        $sucess = null;

        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitAplazoModule')) == true) {
            try {
                $this->postProcess();
                $sucess = Configuration::get('APLAZO_TOKEN_SUCCESS', false);
            } catch (Exception $e) {
                $sucess = false;
                Configuration::updateValue("APLAZO_MERCHANT_ID", null);
                Configuration::updateValue("APLAZO_API_TOKEN", null);
            }
        }

        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl', [
            'module_dir' => $this->_path,
            'sucess'  =>  $sucess,
        ]);

        return $output . $this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitAplazoModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        $this->success[] = $this->l('Information successfully updated.');

        return array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'col' => 5,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-user"></i>',
                        'name' => 'APLAZO_MERCHANT_ID',
                        'label' => $this->l('MERCHANT ID'),
                    ),
                    array(
                        'col' => 5,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-lock"></i>',
                        'name' => 'APLAZO_API_TOKEN',
                        'label' => $this->l('API TOKEN'),
                    )
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'APLAZO_MERCHANT_ID' => Configuration::get('APLAZO_MERCHANT_ID', null),
            'APLAZO_API_TOKEN' => Configuration::get('APLAZO_API_TOKEN', null),
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        /**
         * Validate credentials
         */
        $merchant_id = str_replace(' ', '', Tools::getValue('APLAZO_MERCHANT_ID'));
        $token = str_replace(' ', '', Tools::getValue('APLAZO_API_TOKEN'));
        $service = new AplazoService();

        $service->login($merchant_id, $token);

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, str_replace(' ', '', Tools::getValue($key)));
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        $this->context->controller->addJquery();
        $this->context->controller->addJS($this->_path . 'views/js/back.js?version=' . $this->version, true);
        $this->context->controller->addCSS($this->_path . 'views/css/back.css?version=' . $this->version, true);
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path . 'views/js/front.js', 'all');
        $this->context->controller->addCSS($this->_path . 'views/css/front.css', 'all');
    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        if (!Configuration::get('APLAZO_TOKEN_SUCCESS', false)) {
            return;
        }


        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $cart = $params['cart'];
        $total = (float) $cart->getOrderTotal(true, Cart::BOTH);
        $amount = $total / 5;
        $logo = _PS_BASE_URL_ . __PS_BASE_URI__ . "modules/aplazo/views/img/logo-aplazo.png";

        $titles = [];
        for ($i = 1; $i <= 4; $i++) {
            $_t =  strtotime("+" . ($i * 15) . " day");
            array_push($titles, date('d', $_t) . ' ' . $this->getMonths()[date('m', $_t) - 1]);
        }

        $paramsTemplate = [
            'total' => $total,
            'amount' => $amount,
            'titles' => $titles
        ];

        $output = $this->context->smarty->fetch(
            $this->local_path . 'views/templates/hook/payment_options.tpl',
            $paramsTemplate
        );

        $option = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();

        $option->setModuleName("aplazo")
            ->setCallToActionText(
                $this->l('5 bi weekly installments of ') . ' ' . Tools::displayPrice($amount)  . " con "
            )
            ->setAdditionalInformation($output)
            ->setLogo($logo)
            ->setAction($this->context->link->getModuleLink($this->name, 'redirect', array(), true));

        return [
            $option
        ];
    }


    /**
     * This method is used to render the payment button,
     * Take care if the button should be displayed or not.
     */
    public function hookPayment($params)
    {
        if (!$this->active) {
            return;
        }

        if (!Configuration::get('APLAZO_TOKEN_SUCCESS', false)) {
            return;
        }


        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $cart = $params['cart'];
        $total = (float) $cart->getOrderTotal(true, Cart::BOTH);
        $amount = $total / 5;
        $logo = _PS_BASE_URL_ . __PS_BASE_URI__ . "modules/aplazo/views/img/logo-aplazo.png";

        $titles = [];
        for ($i = 1; $i <= 4; $i++) {
            $_t =  strtotime("+" . ($i * 15) . " day");
            array_push($titles, date('d', $_t) . ' ' . $this->getMonths()[date('m', $_t) - 1]);
        }


        $this->smarty->assign(
            array(
                'total' => $total,
                'logo' => $logo,
                'amount' => $amount,
                'titles' => $titles,
                'this_path' => $this->_path,
                'this_path_bw' => $this->_path,
                'this_path_ssl' => Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ .
                    'modules/' . $this->name . '/'
            )
        );



        return $this->display(__FILE__, 'payment.tpl');
    }


    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookDisplayAdminOrderRight($param)
    { 
        return $this->loadOrderDetailInfo($param, "views/templates/hook/display_admin_order.tpl");
    }


    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookDisplayAdminOrderSide($param)
    {
        return $this->loadOrderDetailInfo($param, "views/templates/hook/display_admin_order.tpl");
    }

    /**
     *
     */
    public function loadOrderDetailInfo($param, $template)
    {
        $order = null;
        if (array_key_exists("order", $param)) {
            $order = $param["order"];
        }
        if (array_key_exists("id_order", $param)) {
            $order = new Order($param["id_order"]);
        }

        $url = $this->context->link->getAdminLink(
            'CancelAplazo',
            true,
            null,
            array(
                'vieworder' => 'cancel',
                'id_order' => $order->id,
                'ajax' => true,
                'action' => 'actionCancelOrder'
            )
        );
        $cancel_id = ApalzoCancelPaymentModel::getIdCancelFromOrderID($param["id_order"]);
        $cancel =  new ApalzoCancelPaymentModel($cancel_id);
        // die(" --------------------------------------------".json_encode($cancel));
        $this->context->smarty->assign("cancel", $cancel);
        $this->context->smarty->assign("order", $order);
        $this->context->smarty->assign("url", $url);
        return $this->display(__FILE__, $template);
    }

    /**
     *
     */
    public function getMonths()
    {
        $months = [
            $this->l('of January'),
            $this->l('of February'),
            $this->l('of March'),
            $this->l('of April'),
            $this->l('of May'),
            $this->l('of June'),
            $this->l('of July'),
            $this->l('of August'),
            $this->l('of September'),
            $this->l('of October'),
            $this->l('of November'),
            $this->l('of December'),
        ];
        return $months;
    }
}
