<?php

/**
 * 2007-2022 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2022 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class AplazoService
{
    /**
     * @var HttpAplazo
     */
    public $httpAplazo;

    /**
     *
     */
    public $tokenCart;

    /**
     *
     */
    public function __construct($setHeader = false)
    {
        $this->httpAplazo = new HttpAp("https://api.aplazo.mx");
        // $this->httpAplazo = new HttpAp("https://api.aplazo.net");
        if ($setHeader) {
            $this->setHeaders();
        }
    }

    /**
     * @param string $merchantId
     * @param string $apiToken
     * @return
     */
    public function login($merchantId, $apiToken)
    {
        try {
            $response = $this->httpAplazo->post(
                'api/auth',
                json_encode(compact('merchantId', 'apiToken')),
                [
                    'headers' => [
                        'Content-Type: application/json',
                        'Accept: application/json',
                    ]
                ]
            );
            $body = $response->getBody();
            $authorization = $body['Authorization'];
            Configuration::updateValue("APLAZO_TOKEN_SUCCESS", true);
            Configuration::updateValue("APLAZO_TOKEN", $authorization);
        } catch (HttpApClientError $error) {
            LogAp::error($error);
            $body = $error->body;
            Configuration::updateValue("APLAZO_TOKEN_SUCCESS", false);
            Configuration::updateValue("APLAZO_TOKEN", null);
            throw $error;
        }
    }

    /**
     * @param string $merchantId
     * @param string $apiToken
     * @return object { [access_token], [expires_in] }
     */
    public function createLoan(&$cart)
    {
        $customer = new Customer($cart->id_customer);
        $addressDelivery = new Address($cart->id_address_invoice);
        $total = (float) $cart->getOrderTotal(true, Cart::BOTH);

        $products = array_map(
            function (&$item) {
                $images = Product::getCover($item['id_product']);
                $image_url = Context::getContext()->link->getImageLink(
                    $item['link_rewrite'],
                    $images['id_image'],
                    version_compare(_PS_VERSION_, '1.7', '<')  ?  null :  ImageType::getFormattedName('home')
                );
                return [
                    "id" => $item['id_product'],
                    "count" => $item['quantity'],
                    "description" => strip_tags(AplazoService::clean($item['description_short'])),
                    "imageUrl" => $image_url,
                    "price" => $item['price_wt'],
                    "title" =>  $item['name'],
                ];
            },
            $cart->getProducts()
        );
        // die(json_encode($products));
        $body = [
            "shopId" => $cart->id_shop,
            "cartId" => "" . $cart->id,
            "buyer" => [
                "addressLine" => $this->getAddressLine($addressDelivery),
                "email" => $customer->email,
                "firstName" => $customer->firstname,
                "lastName" =>  $customer->lastname,
                "loan_id" => 0,
                "phone" => str_replace('+', '', $addressDelivery->phone),
                "postalCode" => $addressDelivery->postcode
            ],
            "products" => $products,
            "discount" => [
                "price" => 0,
                "title" => "not defined"
            ],
            "shipping" => [
                "price" => '0',
                "title" => "not defined"
            ],
            "taxes" => [
                "price" => 0,
                "title" => "not defined"
            ],
            "totalPrice" => $total,
            "successUrl" => $this->successUrl($cart->id),
            "errorUrl" =>  $this->errorUrl($cart->id),
            "webHookUrl" =>  $this->webhookurl($cart->id),
            "cartUrl" =>  $this->cartUrl($cart->id),
        ];

        // LogAp::info('APLAZO   :' . json_encode($body));

        $response = $this->httpAplazo->post(
            'api/loan',
            json_encode($body)
        );
        $body = $response->getBody();

        // LogAp::info('APLAZO LOAN  :' . json_encode($body));

        $this->tokenCart->id_loan = $body['loanId'];
        $this->tokenCart->save();
        return $body['url'];
    }


    /**
     * 
     */
    public function refund($cartId, $reason, $totalAmount)
    {
        $body = [
            "cartId" => $cartId,
            "reason" => $reason,
            "totalAmount" => $totalAmount
        ];

        $merchant_id = Configuration::get('APLAZO_MERCHANT_ID');
        $apiToken = Configuration::get('APLAZO_API_TOKEN');

        $response = $this->httpAplazo->post(
            'api/pos/loan/refund',
            json_encode($body),
            [
                'headers' => [
                    'merchant_id: ' . $merchant_id,
                    'api_token: ' . $apiToken
                ]
            ]
        );
        $body = $response->getBody();

        return $body;
    }


    public static function clean($str)
    {
        $unwanted_array = [
            'Š' => 'S', 'š' => 's', 'Ž' => 'Z', 'ž' => 'z', 'À' => 'A', 'Á' => 'A',
            'Â' => 'A', 'Ã' => 'A', 'Ä' => 'A', 'Å' => 'A', 'Æ' => 'A', 'Ç' => 'C',
            'È' => 'E', 'É' => 'E', 'Ê' => 'E', 'Ë' => 'E', 'Ì' => 'I', 'Í' => 'I',
            'Î' => 'I', 'Ï' => 'I', 'Ñ' => 'N', 'Ò' => 'O', 'Ó' => 'O', 'Ô' => 'O',
            'Õ' => 'O', 'Ö' => 'O', 'Ø' => 'O', 'Ù' => 'U', 'Ú' => 'U', 'Û' => 'U',
            'Ü' => 'U', 'Ý' => 'Y', 'Þ' => 'B', 'ß' => 'Ss', 'à' => 'a', 'á' => 'a',
            'â' => 'a', 'ã' => 'a', 'ä' => 'a', 'å' => 'a', 'æ' => 'a', 'ç' => 'c',
            'è' => 'e', 'é' => 'e', 'ê' => 'e', 'ë' => 'e', 'ì' => 'i', 'í' => 'i',
            'î' => 'i', 'ï' => 'i', 'ð' => 'o', 'ñ' => 'n', 'ò' => 'o', 'ó' => 'o',
            'ô' => 'o', 'õ' => 'o', 'ö' => 'o', 'ø' => 'o', 'ù' => 'u', 'ú' => 'u',
            'û' => 'u', 'ý' => 'y', 'þ' => 'b', 'ÿ' => 'y', '"' => '', '\u00a0' => ''
        ];
        return strtr(str_replace("\u{00a0}", ' ', $str), $unwanted_array);
    }

    /**
     *
     */
    public function webhookurl($cart_id)
    {
        $this->tokenCart = ApalzoTokenCartModel::createToken($cart_id);
        return Context::getContext()->link->getModuleLink(
            "aplazo",
            "process",
            ['aplazo_token' => $this->tokenCart->token, 'cart_id' => $cart_id]
        );
    }

    /**
     *
     */
    public function cartUrl()
    {
        return Context::getContext()->link->getPageLink('order', null, null, 'step=3');
    }

    /**
     *
     */
    public function successUrl($cart_id)
    {
        return Context::getContext()->link->getModuleLink("aplazo", "success", ['cart_id' => $cart_id]);
    }

    /**
     *
     */
    public function errorUrl($cart_id)
    {
        return Context::getContext()->link->getModuleLink("aplazo", "fail", ['cart_id' => $cart_id]);
    }

    /**
     * @return String
     */
    public function getAddressLine(&$address)
    {
        $state = new State($address->id_state);
        $arr = [
            $address->country,
            $state->name,
            $address->city,
            $address->address1,
            $address->address2,
        ];

        return join(
            ', ',
            array_filter(
                $arr,
                function ($item) {
                    return !empty($item);
                }
            )
        );
    }

    /**
     * @return AplazoService
     */
    public function setHeaders()
    {
        $this->httpAplazo->setRequestHeaders($this->getHeaders());
        return $this;
    }

    /**
     * @param string $clinet
     * @param string $secret
     * @return array { token }
     */
    public function getHeaders()
    {
        $merchant_id = Configuration::get('APLAZO_MERCHANT_ID');
        $apiToken = Configuration::get('APLAZO_API_TOKEN');
        $this->login($merchant_id, $apiToken);
        $token = Configuration::get('APLAZO_TOKEN');
        $headers = [
            'Content-Type: application/json',
            'Accept: application/json',
        ];
        if ($token) {
            array_push($headers, 'Authorization: ' . $token);
        }
        return $headers;
    }
}
